/*
Programa para Arduino como visualizador externo de un FT857.
Basado en un programa de IZ0MJE Stefano Sinagra para el FT817
publicado bajo licencia GNU/GPL en su versión 0.01 el 18 de
junio de 2001
Modificaciones realizadas por Jon Iza EA2SN, Mayo-Junio 2020
Primera versión desde el IZ0MJE.PDE para actualizar librerías.
- Usaba NewSoftSerial que está integrada en las versiones
más modernas de Arduino como SoftwareSerial: Ha habido que
cambiar el original   
 mySerial.print(catCmd[i], BYTE);
por
 CAT.write(catCmd[i]);
ya que BYTE quedó obsoleto.
- Cambiado el patillaje del LCD usando una shield LCD con 
pulsadores
En pantalla muestra
A  145.600.00  R|    VFO  Frecuencia     Antena F/R
FM    S:00   P4 |    Modo  Smeter 00-15  Potencia 1-4
-El análisis del sketch lleva a que se usan las líneas 2 y 3
para la comunicación de datos (TxD y RxD). Los datos los saca
por interrogación con un comando CAT.

======================================================
CONEXIONES
Arduino UNO alimentado por USB 5 V
LCD Keypad Shield
Cable con conector miniDIN de 8 polos (visto por detrás
   Fila central     RxD  HUECO  TxD  GND
   Arduino patillas 3            2   GND
======================================================
*/

#include <SoftwareSerial.h>
SoftwareSerial CAT(2, 3); // TxD y RxD del CAT al conector MiniDIN
char catCmd[] = { 00, 00, 00, 00, 00 } ; // formato de 5 bytes del comando CAT
char rxreply[2];

static int pause = 25;
byte vband[2];
byte fant;
byte antenna;
byte vfo;
static byte bitshift[15] = {7,7,7,7,7,7,7,7,7,6,5,4,3,2,7};
byte power;


// librería de manejo del visualizador:
#include <LiquidCrystal.h>
// inicializar con las patillas del LCD
LiquidCrystal lcd( 8,9,4,5,6,7);


void setup()  
{
  // Salida serie con leyenda sobre autoría
  // No queda claro si es GNU/GPL por qué incluye copyright
  //Serial.begin(57600);
  //Serial.println("Arduino & FT817");
  //Serial.println("(C) 2011 Stefano 'Sinager' Sinagra IZ0MJE");
  //Serial.println("released under GNU/GPL licence");

  // ajuste de velocidad de comunicación CAT 9600 baudios
  CAT.begin(9600);

  // inicialización del visualizador LCD
  lcd.begin(16, 2);
}

void ftCmd()
{
  // envío del comando de interrogación CAT 
  for (int i=0; i<5; i++){
    CAT.write(catCmd[i]);
    }
}

void printfreq(long int freq) 
{
  // formateado de frecuencia para el visualizador
  long int fmhz = freq/100000;
  long int fkhz = (freq - (fmhz * 100000))/100;
  long int fdec = freq - (fmhz * 100000) - fkhz*100;
  // linea superior posición 4  3 dígitos.3dígitos.2dígitos
  lcd.setCursor(3, 0);
  padzeromega(fmhz);
  lcd.print(fmhz, DEC);
  lcd.print(".");
  padzerokilo(fkhz);
  lcd.print(fkhz, DEC);
  lcd.print(".");
  if (fdec < 10){lcd.print("0");}
  lcd.print(fdec, DEC);
}

int padzeromega(int digits)
{
  if (digits == 0 ){lcd.print("  ");
    } else if ((digits < 10) && (digits > 0) ){lcd.print("  ");
    } else if ((digits >= 10) && (digits < 100) ){lcd.print(" ");}
}

int padzerokilo(int digits)
{
  if (digits == 0 ){lcd.print("00");
    } else if ((digits < 10) && (digits > 0) ){lcd.print("00");
    } else if ((digits >= 10) && (digits < 100) ){lcd.print("0");}
}
void smeter()
{
  // comando CAT para preguntar Smeter  00-00-00-00-E7
  // resultado 8 bits, últimos 4 datos de Smeter
  delay(pause);
  catCmd[0] = 0;
  catCmd[1] = 0;
  catCmd[2] = 0;
  catCmd[3] = 0;
  catCmd[4] = 0xe7 ;
  ftCmd();
  delay(pause);
   // lectura de bytes
   int i = 0;
   while ( i < 1 ){
      if (CAT.available()) {
        int rxreply = CAT.read();
        byte bitsmtr = rxreply << 4;
        bitsmtr = bitsmtr >> 4;
          // linea inferior posición 7 S más dos cifras "bitsmtr" 7-8-9
        lcd.setCursor(6, 1);
        lcd.print("S");                    
        if (bitsmtr < 10) {lcd.print("0");}
        lcd.print(bitsmtr, DEC); 
        i++;
        }
    }
}

void rdband()
{
  // comando CAT para preguntar Banda 00-59-00-00-BB
  // no se conoce formato
  delay(pause);
  catCmd[0] = 0;
  catCmd[1] = 0x59;
  catCmd[2] = 0;
  catCmd[3] = 0;
  catCmd[4] = 0xbb;
  ftCmd();
  delay(pause);
  // lectura de bytes
  byte rxreply[2];
  int i = 0;
   while ( i < 2 ){
      if (CAT.available()) {
          rxreply[i] = CAT.read();
          vband[1] = rxreply[0] << 4;
          vband[1] = vband[1] >> 4;
          vband[0] = rxreply[0] >> 4;
            i++;
        }
    }
   // vband se usa para saber el conector de antena
}

void rdpwr()
{
  // comando CAT para preguntar Potencia 00-79-00-00-BB
  // no se conoce formato
  delay(pause);
  catCmd[0] = 0;
  catCmd[1] = 0x79;
  catCmd[2] = 0;
  catCmd[3] = 0;
  catCmd[4] = 0xbb;
  ftCmd();
  delay(pause);
  // lectura de bytes
  byte rxreply[2];
  int i = 0;
   while ( i < 2 ){
      if (CAT.available()) {
          rxreply[i] = CAT.read();
            i++;
        }
    }
  power = rxreply[0] << 6 ;
  power = 4 - (power >> 6) ;
    // linea inferior posición 14 P más dos cifras "power" 14-15
  lcd.setCursor(13, 1);
  lcd.print("P");
  lcd.print(power, DEC);
}



void readant()
{
  // comando CAT para preguntar Conector de antena 00-7A-00-00-BB
  // no se conoce formato
  delay(pause);
  catCmd[0] = 0;
  catCmd[1] = 0x7a;
  catCmd[2] = 0;
  catCmd[3] = 0;
  catCmd[4] = 0xbb;
  ftCmd();
  delay(pause);
  // lectura de bytes
  byte rxreply[2];
  int i = 0;
   while ( i < 2 ){
      if (CAT.available()) {
          rxreply[i] = CAT.read();
            i++;
        }
    }
  antenna = rxreply[0];
    // linea superior posición 16 una sola letra F o R
  lcd.setCursor(15, 0);
  antenna = antenna << bitshift[vband[vfo]];
  antenna = antenna >> 7;
  if (antenna == 0){lcd.print("F");} else {lcd.print("R");}
}


void readvfo()
{
  // comando CAT para preguntar VFO (A o B) 00-55-00-00-BB
  // no se conoce formato
  delay(pause);
  catCmd[0] = 0;
  catCmd[1] = 0x55;
  catCmd[2] = 0;
  catCmd[3] = 0;
  catCmd[4] = 0xbb ;
  ftCmd();
  delay(pause);
  // lectura de bytes
  int i = 0;
   while ( i < 2 ){
      if (CAT.available()) {
        rxreply[i] = CAT.read();
        i++;
        }
    }
        vfo = rxreply[0] & 1;
        vfo = ! vfo;
        // linea superior posición 1 una sola letra A o B
        lcd.setCursor(0, 0);
        if (vfo == 0) {
              lcd.print("B");
          } else {lcd.print("A");}
}


void ftRply()
{
  // comando CAT para preguntar Frecuencia y modo 00-00-00-00-03
  // primero va frecuencia y luego modo
  // MM-Mk-kk-HzHz-MODO
  delay(pause);
  catCmd[0] = 0;
  catCmd[1] = 0;
  catCmd[2] = 0;
  catCmd[3] = 0;
  catCmd[4] = 0x03 ;
  ftCmd();
  delay(pause);
    // lectura de bytes
    unsigned long i = 10000000;
    char hmode;
    String mode;
    unsigned long freq = 0;
    int z=0;
    while ( i > 1){
      if (CAT.available()) {
        int rxreply = CAT.read();
        rxreply = (rxreply / 16) * 10 + rxreply % 16;      
        freq = (freq + rxreply * i);
        i = i/100;
        }
      }
    while (z<1){
      if (CAT.available()) {
        hmode = CAT.read();
        z++;
        } 
      } 
    freq = freq / 10;
    //formato de modo: 00 - LSB, 01 - USB, 02 - CW, 03 - CWR, 
    // 04 - AM, 06 - WFM, 08 - FM, 0A - DIG, 0C – PKT
      switch (hmode) {
    case 0x00:
      mode = "LSB";
      break;
    case 0x01:
      mode = "USB";
      break;
    case 0x02:
      mode = "CW ";
      break;
    case 0x03:
      mode = "CWR";
      break;
    case 0x04:
      mode = "AM ";
      break;
    case 0x06:
      mode = "WFM";
      break;
    case 0x08:
      mode = "FM ";
      break;
    case 0x0a:
      mode = "DIG";
      break;
    case 0x0c:
      mode = "PKT";
      break;
    default: 
      mode = "ERR";
      }
    // primero presenta frecuencia (ya tiene su formato)
    printfreq(freq);
    // linea inferior posición 1 tres caracteres 1-2-3   
    lcd.setCursor(0, 1);    
    lcd.print(mode);
    }   



void loop()
{
  ftRply();  // leer frecuencia 
  smeter();  // leer smeter
  readvfo(); // leer VFO (A/B)
  rdband();  // leer la banda (para conocer la antena)
  readant(); // leer la antena
  rdpwr();   // leer la potencia
}
